package ASCOPE::Article::Collection;
use strict;

$ASCOPE::Article::Collection::VERSION = '1.3';

# $Id: Collection.pm,v 1.12 2004/07/23 12:59:59 asc Exp $

=head1 NAME

ASCOPE::Article::Collection - Aaron's widget for munging a specialized subset of the DocBook format

=head1 SYNOPSIS

 use ASCOPE::Article::Collection;
 my $collection = ASCOPE::Article::Collection->new("/path/to/articles.xml");

 $collection->mk_html({htroot => "/htdocs/news",
                       uri    => "http://127.0.0.1/news",
                       flush  => 1});

=head1 DESCRIPTION

Aaron's widget for munging a specialized subset of the DocBook format used to
record one or more articles.

=head1 DOCUMENT STRUCTURE

Book widgets contain the following elements :

 + book
   + bookinfo
     - title
     - pubdate
   + part (+)
     + partinfo
       - title
     + xref (+)
       @linkend
     + article (+)

Article widgets contain the following elements :

 + article
   @id
   + articleinfo
     - title
     + keywordset (or not)
       - keyword
     + indexterm (or not)
       @ type
       - primary
     + authorgroup
       - corpauthor
       OR
       + author
        - surname
        - firstname
        - othername (or not)
     + abstract
       - para
     - pubdate
     + copyright (or not, but should be)
       - year
       - holder
   - para (+)

=head1 TOPICS, KEYWORDS, INDEXTERMS AND RELATIONS

Individual articles may contain one or more I<keywordset> or I<indexterm>
thingies. When these relations are indexed in any of the methods below,
they will be returned with the following labels :

=head2 keywords

A keyword is labeled as an "idea".

=head2 indexterms

Index terms are labeled as follows - where the type attribute is :

=over 4

=item * B<org>

"organization"

=item * B<geo>

"place"

=item * B<per>

"person"

=back

=cut

use ASCOPE::Article::Collection::Constants qw (:rels);

use XML::LibXML;
use File::Path;
use File::Spec;
use IO::File;
use CGI;
use DirHandle;
use Memoize;
use Digest::MD5 qw (md5_hex);

&memoize("_normalize");

=head1 PACKAGE METHODS

=cut

=head2 __PACKAGE__->new($path)

Returns a I<ASCOPE::Article::Collection> object. Woot!

=cut

sub new {
  my $pkg  = shift;
  my $path = shift;

  my $parser = XML::LibXML->new();
  my $doc    = $parser->parse_file($path);

  return bless {'__doc'  => $doc,
		'__path' => $path}, $pkg;
}

=head1 OBJECT METHODS

=cut

sub path {
    return $_[0]->{'__path'};
}

=head2 $obj->pubdate()

Returns a string.

=cut

sub pubdate {
  my $self = shift;
  $self->{'__date'} ||= $self->{'__doc'}->find(qq(/book/bookinfo/pubdate));
  return $self->{'__date'};
}

=head2 $pkg->mk_html(\%args)

=over 4

Render a collection of articles as an HTML tree.

=item * B<htroot>

The root path to a directory where files are created.

=item * B<uri>

String. 

The root URI for links.

=item * B<dated_space>

Boolean.

Indicated whether files should be created under I<htroot>
or I<htroot>/YYYY/MM/DD

=item * B<flush>

Boolean.

Remove old files and directories.

=item * B<verbose>

Boolean.

=back

Returns true or false.

=cut

sub mk_html {
  my $self = shift;
  my $args = shift;

  my $htroot = $args->{htroot};
  my $uri    = $args->{uri};

  #

  if ($args->{dated_space}) {
      $self->pubdate()   =~ m!^(\d{4})(?:-|/)(\d{2})(?:-|/)(\d{2})!;
      my ($yyyy,$mm,$dd) = ($1,$2,$3);

      $htroot = File::Spec->catdir($htroot,$yyyy,$mm,$dd);
      $uri    = "$uri/$yyyy/$mm/$dd";
  }

  #

  if ((-d $htroot) && ($args->{flush})) {
    my $dh = DirHandle->new($htroot);

    foreach my $subdir ($dh->read()) {
      next if ($subdir =~ /^\./);
      next if ($subdir =~ /^\d{4}$/);

      if (! rmtree("$htroot/$subdir", $args->{verbose})) {
	  warn sprintf("failed to rmtree '%s', %s",$htroot,$!);
	  return undef;	  
      }
    }

  }

  elsif (! mkpath($htroot,$args->{verbose})) {
      warn sprintf("failed to mkdir '%s', %s",$htroot,$!);
      return undef;
  }

  else {}

  #

  $self->{'__cgi'} ||= CGI->new();

  # Create an index page for the day 

  $self->_mk_html($htroot,$uri);
  return 1;
}

sub _mk_html {
    my $self   = shift;

    my $htroot = shift;
    my $uri    = shift;

    my $cgi   = $self->{'__cgi'};
    my $title = $self->{'__doc'}->find(qq(/book/bookinfo/title));
    
    my $path_index = File::Spec->catfile($htroot,"index.html");
    my $index      = IO::File->new(">$path_index");
    
    $index->print($cgi->start_html(-title=>$title));
    $index->print($cgi->h1($title));
    
    #
    
    foreach my $section ($self->{'__doc'}->findnodes("/book/part")) {
	$self->_mk_html_sect($htroot,$uri,\*$index,$section);
    }

    # Finish up the index page
    
    $index->print($cgi->end_html());
    $index->close();
    
    return 1;
}

sub _mk_html_sect {
    my $self    = shift;

    my $htroot  = shift;
    my $uri     = shift;

    my $index   = shift;
    my $section = shift;

    # The widget that generates the DocBook is SAX
    # driven and generates a 'part' (section) event
    # even if there are no articles for that date.

    my @articles = $section->findnodes(qq(child::node()[name() = 'article' or name() = 'xref']));

    if (! @articles) {
	return 1;
    }

    # Munge the section title

    my $sectname = $section->find(qq(partinfo/title));
    $self->_sectname(\$sectname);
    
    # Create directory root in which
    # we'll write articles.

    my $root_sect = File::Spec->catdir($htroot,$sectname);    

    if ((! -d $root_sect) && (! mkpath([$root_sect],0,0755))) {
	warn "failed to mkdir '$root_sect', $!";
	next;
    }

    # More happy HTML

    my $cgi = $self->{'__cgi'};

    $index->print($cgi->h2($cgi->a({href=>"$uri/$sectname"},
				   $sectname)));

    # Create an index page for this section

    my $path_sect = File::Spec->catfile($root_sect,"index.html");
    my $sect      = IO::File->new(">$path_sect");

    if (! $sect) {
	warn "failed to open '$path_sect' for writing, $!";
	next;
    }

    my $title = "FIX ME";

    $sect->print($cgi->start_html(-title=>"$title $sectname"));
    $sect->print($cgi->h1($sectname));

    foreach my $article (@articles) {

	$self->_mk_html_article($htroot,$uri,
				\*$index,\*$sect,
				$sectname,$article);
    }

    # Finish up the index page
    # for this section.

    $sect->print($cgi->end_html());
    $sect->close();
    return 1;
}

sub _mk_html_article {
    my $self    = shift;

    my $htroot  = shift;
    my $uri     = shift;

    my $index   = shift;
    my $sect    = shift;

    my $sectname = shift;
    my $article = shift;

    # If we're dealing with an xref then we will
    # need to make sure that the parent ($sectname)
    # for the article points to the source and not
    # the current section we're iterating through.
    
    # Rather than do some fancy-pants local() kung-fu
    # we'll just set up a 'parent' variable for this
    # block and get on with it.
    my $parent =  $sectname;
    
    if ($article->nodeName() eq "xref") {
	my $ref = $article->getAttribute(qq(linkend));
	
	$article = ($self->{'__doc'}->findnodes(qq(*//article[\@id='$ref'])))[0];
	
	if (! $article) {
	    warn "Unable to find article: $ref\n";
	    next;
	}
	
	# Update the 'parent' so that filenames and
	# uris are generated correctly and point to
	# the source article
	
	$parent = $article->find(qq(parent::node()/partinfo/title));
	$self->_sectname(\$parent);
    }

    my $cgi = $self->{'__cgi'};
    
    # Get the ID for the article (this is actually
    # an MD5 hex of the uri for the article.
    
    my $id  = $article->getAttribute(qq(id));
    my $md5 = $self->_md5($id);
    
    # Pull out the articleinfo and create some
    # variables that we'll use below.
    
    my $info = $self->_articleinfo($article);
    
    if (! $info) {
	$sect->print($cgi->p("Unable to find any articleinfo for '$id'\n"));
	next;
    }
    
    #
    
    my $title  = $info->find(qq(title));
    my $author = $self->_author($info);
    
    $uri = "$uri/$parent/$md5.html";
    
    # List the article on the main page.
    
    $index->print($cgi->a({href=>$uri},
			  $title));

    $index->print("&nbsp;$author");
    $index->print($cgi->br());
    
    # List the article and its abstract
    # on the section page.
    
    $sect->print($cgi->a({href=>$uri},
			 $title));

    $sect->print($cgi->br());
    $sect->print($cgi->p($info->find(qq(abstract/para))));
    
    # Create the actual article HTML page.
    # Assuming that we're not dealing with
    # an reference, in which case it's already
    # been written to disk.
    
    my $root_text = File::Spec->catdir($htroot,$parent);
    my $path_text = File::Spec->catfile($root_text,"$md5.html");
    
    if (! -f $path_text) {
	
	my $text = IO::File->new(">$path_text");
	
	if (! $text) {
	    warn "failed to open '$path_text' for writing, $!";
	    next;
	}
	
	$text->print($cgi->start_html(-title=>$title),
		     $cgi->h1($title),
		     $cgi->p($author));
	
	$self->_mk_html_article_body(\*$text,$article);

	$text->print($cgi->a({href=>$id},$id));
	$text->print($cgi->end_html());
	$text->close();
    }

    return 1;
}

sub _mk_html_article_body {
    my $self    = shift;

    my $text    = shift;
    my $article = shift;

    my $cgi = $self->{'__cgi'};

    map {
	$text->print($cgi->p($_));
    } @{ $self->_cleanarticle($article) };

    return 1;
}

=head2 $obj->index_plucene(Plucene::Simple)

Index a collection with I<Plucene::Simple>.

Returns true or false.

=cut

sub index_plucene {
  my $self = shift;
  my $idx  = shift;

  #

  $self->pubdate() =~ /^([-\d]+)/;
  my $date = $1;

  #
  
  foreach my $section ($self->{'__doc'}->findnodes("/book/part")) {
      
      my @articles = $section->findnodes(qq(child::node()[name() = 'article' or name() = 'xref']));
      next unless (@articles);
      
      # Munge the section title
      
      # my $sectname = $section->find(qq(partinfo/title));
      # $self->_sectname(\$sectname);
   
      # This needs to be be massaged to do something
      # more intelligent with referenced articles.

      foreach my $article (grep {$_->nodeName() ne "xref"} @articles) {

	  my $id   = $self->_uid($article);
	  my $seen = 0;

	  # I should submit a bug report
	  # about this - Plucene will die
	  # gracelessly if the index doesn't
	  # exist yet

	  eval {
	      $seen = $idx->indexed($id);
	  };

	  if ($seen) {
	      next;
	  }

	  #

	  my $data        = $self->hr_articleinfo($article);
	  $data->{ text } = join("\n",@{ $self->_cleanarticle($article) });

	  eval { 
	      $idx->add( $id => $data); 
	  };

	  if ($@) {
	      warn sprintf("[%s] failed to index %s, %s",$id,$data->{title},$@);
	  }
      }
  }

  return 1;
}

=head2 $obj->index_contextgraph(Search::ContextGraph)

Index a collection with I<Search::ContextGraph>.

=cut

sub index_contextgraph {
  my $self  = shift;
  my $graph = shift;

  require File::Temp;

  #

  foreach my $section ($self->{'__doc'}->findnodes("/book/part")) {
      
      my @articles = $section->findnodes(qq(child::node()[name() = 'article' or name() = 'xref']));
      next unless (@articles);
   
      # This needs to be be massaged to do something
      # more intelligent with referenced articles.

      foreach my $article (grep {$_->nodeName() ne "xref"} @articles) {

	  my $id  = $self->_uid($article);

	  if ($graph->has_doc($id)) {
	      next;
	  }

	  #

	  my ($fh,$fn) = File::Temp::tempfile(undef,UNLINK=>0);
	  $fh->print(join("\n",@{ $self->_cleanarticle($article) }));
	  $fh->close();

	  $graph->add_file($fn,name=>$id);
	  unlink($fn);
      }

      last;
  }

  return 1;
}

=head2 $obj->index_delicious(ASCOPE::Delicious)

Index a collection with I<ASCOPE::Delicious>.

=cut

sub index_delicious {
    my $self      = shift;
    my $delicious = shift;

    #

    foreach my $section ($self->{'__doc'}->findnodes("/book/part")) {

	my @articles = $section->findnodes(qq(child::node()[name() = 'article' or name() = 'xref']));
	next unless (@articles);

	foreach my $article (grep {$_->nodeName() ne "xref"} @articles) {
	  
	    my $id   = $article->getAttribute(qq(id));
	    my $info = $self->_articleinfo($article);

	    if (! $info) {
		next;
	    }

	    my $title    = $info->find(qq(title));
	    my $abstract = $info->find(qq(abstract/para));

	    my @tags = ();

	    #

	    foreach my $term (RELS_INDEXTERMS) {
		map {
		    push @tags, join("/",RELS_LABELS->{$term},
				         $self->normalize($_));
		} grep { 
		    $_ =~ /\w/;
		} $self->_indexterm($info,$term);
	    }
	    
	    foreach my $kw (RELS_KEYWORDS) {
		map { 
		    push @tags, join("/",RELS_LABELS->{$kw},
				         $self->normalize($_));
		} grep { 
		    $_ =~ /\w/;
		} $self->_keywords($info);
	    }

	    #

	    if (! @tags) {
		next;
	    }

	    #

	    my $bm = $delicious->add_post({url         => $id,
					   title       => $title,
					   description => $abstract,
					   tag         => join(" ",@tags)});
	}
	
    }

    return 1;
}

=head2 $obj->stats()

Returns a hash reference of relations and their number of referents
organized by topic. e.g. :

 geo => %
     Baltimore (Md) => 1
     Brazil => 1
     Brooklyn (NYC) => 1
     ...
 org => %
     Air Canada => 1
     Al Jazeera => 1
     Al Qaeda => 2

This method shares a degree of overlap with the I<weight_relations>
method that remains to be reconciled.

=cut

sub stats {
  my $self  = shift;
  my %stats = ();

  #
  
  foreach my $section ($self->{'__doc'}->findnodes("/book/part")) {
      
      my @articles = $section->findnodes(qq(child::node()[name() = 'article' or name() = 'xref']));
      next unless (@articles);

      foreach my $article (grep {$_->nodeName() ne "xref"} @articles) {
	  
	  my $info = $self->_articleinfo($article);

	  if (! $info) {
	      next;
	  }

	  #

	  foreach my $term (RELS_INDEXTERMS) {
	      map {  
		  $stats{ $term }->{ $_ } ++;
	      } grep { 
		  $_ =~ /\w/;
	      } $self->_indexterm($info,$term);
	  }

	  foreach my $kw (RELS_KEYWORDS) {
	      map {
		  $stats{ $kw }->{ $_ } ++;
	      } grep { 
		  $_ =~ /\w/;
	      } $self->_keywords($info);
	  }

      }

  }

  return \%stats;
}

=head2 $obj->relations()

Returns a hash ref of all the relations and their topics and
referents. e.g. :

 Johnson, Magic => %
       keywords => %
            Automobile Racing => 1
       org => %
            National Assn of Stock Car Auto Racing => 1
 Leiter, Al => %
       keywords => %
            Baseball => 1
       org => %
            New York Mets => 1

=cut

sub rels { return shift->relations(); }

sub relations {
  my $self = shift;
  my %rels = ();

  #
  
  foreach my $section ($self->{'__doc'}->findnodes("/book/part")) {
            
      my @articles = $section->findnodes(qq(child::node()[name() = 'article' or name() = 'xref']));
      next unless (@articles);

      foreach my $article (grep {$_->nodeName() ne "xref"} @articles) {
	  
	  my %stats = ();
	  my $info  = $self->_articleinfo($article);

	  if (! $info) {
	      next;
	  }

	  #

	  foreach my $term (RELS_INDEXTERMS) {
	      map {  
		  $stats{ $term }->{ $_ } ++;
	      } grep { 
		  $_ =~ /\w/;
	      } $self->_indexterm($info,$term);
	  }

	  foreach my $kw (RELS_KEYWORDS) {
	      map { 
		  $stats{ $kw }->{ $_ } ++;
	      } grep { 
		  $_ =~ /\w/;
	      } $self->_keywords($info);
	  }

	  #

	  foreach my $topic (keys %stats) {
	      foreach my $what (keys %{$stats{$topic}}) {

		  foreach my $_topic (keys %stats) {
		      foreach my $_what (keys %{$stats{$_topic}}) {

			  next if (($_topic eq $topic) && ($_what eq $what));
			  $rels{ $topic }->{$what}->{$_topic}->{$_what} ++;
		      }
		  }
	      }
	  }

	  #

      }
  }

  #

  return \%rels;
}

=head2 $obj->weigh_relations(\%relations)

Returns a hash reference of all topics and the number of
references pointing to each.

  my $relations = $collection->relations();
  my $weights   = $collection->weigh_relations($relations);

The hash reference contains a nested hash whose key is I<__topic>
and which maps topic name to topic type. e.g. :

 %
     Accounting and Accountants => 3
     Air Canada => 2
     Air Pollution => 3
     Airlines and Airplanes => 2
     Al Jazeera => 4
     ...
 __topics => %
     Taxation => keywords 
     ...

This method shares a degree of overlap with the I<stats> method
that remains to be reconciled.

=cut

sub weigh_relations {
    my $self = shift;
    my $rels = shift;

    my %weights = ();

    foreach (keys %$rels) {
	my $topic = $rels->{$_};

	foreach (keys %$topic) {
	    my $what = $topic->{$_};

	    foreach my $rel (keys %$what) {
		my $type = $what->{$rel};

		foreach my $this (keys %$type) {
		    $weights{ $this} ++;
		    $weights{"__topic"}->{ $this } = $rel;		    
		}
	    }
	}
    }

    return \%weights;
}

=head2 $obj->heaviest_relations(\%weights)

Returns an array reference of one, or more, topics with
the highest number of references.

  my $relations = $collection->relations();
  my $weights   = $collection->weigh_relations($relations);
  my $heaviest  = $collection->heaviest_relations($weights);

=cut

sub heaviest_relations {
    my $self    = shift;
    my $weights = shift;

    my %foo = ();

    map {
	push @{ $foo{ $weights->{$_} } }, $_;
    } keys %$weights;
    
    my $i = (sort {$b <=> $a} keys %foo)[0];
    
    #

    return $foo{$i};
}

sub expiliciousize_relations {
    # to do for 1.4 maybe...
    
}

=head2 $obj->graph_relations(GraphViz,\@subset)

Graph all relations, or a user-defined subset of keywords or indexterms 
passed as an array reference.

    my $gv = GraphViz->new(...);

  my $relations = $collection->relations();
  my $weights   = $collection->weigh_relations($relations);
  my $heaviest  = $collection->heaviest_relations($weights);

  $collection->graph_relations($gv,$heaviest);
  $gv->as_svg(...);

=cut

sub graph_relations {
    my $self  = shift;
    my $graph = shift;
    my $slice = shift;
    my $args  = shift;

    $args ||= {};

    if ($slice) {
	return $self->_graph_slice($graph,$slice,$args);
    }

    #

    my $relations = $self->relations();
    my $weights   = $self->weigh_relations($relations);

    #

    foreach my $topic (keys %$relations) {
	foreach my $what (keys %{$relations->{$topic}}) {

	    my %node_args = (fontsize  => ($args->{fontsize}  || 8),
			     shape     => ($args->{shape}     || RELS_SHAPES->{$topic}),
			     fillcolor => ($args->{fillcolor} || RELS_COLOURS->{$topic}),
			     style     => "filled",);

	    if ($args->{nolabels}) {
		$node_args{ label } = "";
	    }
	    
	    $graph->add_node($what,%node_args);
	}
    }

    #

    foreach (keys %$relations) {
	my $topic = $relations->{$_};

	foreach my $node (keys %$topic) {
	    my $what = $topic->{$node};

	    foreach (keys %$what) {
		my $type = $what->{$_};

		foreach my $edge (keys %$type) {

		    $graph->add_edge($node    => $edge,
				     weight   => $weights->{ $edge },
				     samehead => 1,
				     sametail => 1,
				     minlen   => 2);
		}
	    }
	}
    }

    #

    return 1;
}

sub _graph_slice {
    my $self  = shift;
    my $graph = shift;
    my $which = shift;
    my $args  = shift;

    my $rels    = $self->relations();
    my $weights = $self->weigh_relations($rels);

    #

    my %nodes   = ();

    #

    foreach (keys %$rels) {
	my $topic = $rels->{$_};

	foreach my $this (keys %$topic) {

	    # This sometimes fails (dunno) :
	    # next unless (grep /^($this)$/,@$which);

	    # So we are reduced to this :

	    my $ok = 0;

	    foreach (@$which) { 
		if ($this eq $_) {
		    $ok = 1;
		    last;
		}
	    }

	    if (! $ok) {
		next;
	    }

	    #

	    foreach my $rel (keys %{$topic->{$this}}) {
		$nodes{ $this }->{ $rel } = $topic->{$this}->{$rel};
	    }
	}
    }

    #

    foreach my $first (keys %nodes) {

	my %node_args = (fontsize  => 6,
			 shape     => "ellipse",
			 fillcolor => RELS_COLOURS->{ $weights->{'__topic'}->{$first} },
			 style     => "filled");
	
	if ($args->{nolabels}) {
	    $node_args{ label } = "";
	}

	$graph->add_node($first,%node_args);

	foreach my $rel (keys %{$nodes{$first}}) {

	    foreach my $what (keys %{$nodes{$first}->{$rel}}) {

		my %node_args = (fontsize  => 6,
				 shape     => RELS_SHAPES->{$rel},
				 fillcolor => RELS_COLOURS->{$rel},
				 style     => "filled");
		
		if ($args->{nolabels}) {
		    $node_args{ label } = "";
		}

		$graph->add_node($what,%node_args);
	    }
	}
    }

    # 

    foreach my $first (keys %nodes) {
	foreach my $rel (keys %{$nodes{$first}}) {
	    foreach my $edge (keys %{$nodes{$first}->{$rel}}) {

		$graph->add_edge($first   => $edge,
				 samehead => 1,
				 sametail => 1,
				 minlen   => 2);

		foreach my $topic (keys %{$rels->{$rel}->{$edge}}) {

		    foreach my $what (keys %{$rels->{$rel}->{$edge}->{$topic}}) {

			if (exists($nodes{$first}->{$topic}->{$what})) {
			    $graph->add_edge($edge    => $what,
					     samehead => 1,
					     sametail => 1,
					     minlen   => 2);
			}


		    }
		}
	    }
	}
    }

    #

    return 1;
}

=head2 $obj->hr_articleinfo(XML::LibXML::Node)

Utility method to return the I<articleinfo> for an article
as a hash reference.

=cut

sub hr_articleinfo {
    my $self    = shift;
    my $article = shift;

    my %data = ();

    # Pull out the articleinfo and create some
    # variables that we'll use below.
    
    my $info = $self->_articleinfo($article);

    if (! $info) {
	return \%data;
    }

    #

    if (my $date = $info->find(qq(pubdate))) {
	$date =~ /^(\d{4}-\d{2}-\d{2}).*/;
	$data{ date } = $1;
    }

    if (my $title = $info->find(qq(title))) {

	# lots of things do ref() tests on strings and
	# the magic stringy-ify fu  that XML::libXML
	# does on its objects is lost on them

	$title =  $title->to_literal();

	# paranoia / will destroy ya

	if ($title) {
	    $title = $title->value();
	}

	$title =~ s/\n/ /gm;

	$data{ title } = $title;
    }
    
    if (my $author = $self->_author($info)) {
	$data{ author } = "$author";
    }
    
    if (my $person = join(",",$self->_indexterm($info,"per"))) {
	$data{ person } = $person;
    }
    
    if (my $org = join(",",$self->_indexterm($info,"org"))) {
	$data{ org } = $org;
    }
    
    if (my $geo = join(",",$self->_indexterm($info,"geo"))) {
	$data{ geo } = $geo;
    }
    
    if (my $kw = join(",",$self->_keywords($info))) {
	$data{ keywords } = $kw;
    }

    return \%data;
}

sub normalize {
    my $pkg = shift;

    # until I have the time to read the
    # Memoize.pm on passing package 
    # names/refs it's easier just to do
    # this...
    
    return &_normalize($_[0]);
}

sub _normalize {
    my $str = shift;

    $str = lc($str);

    $str =~ s/^\s+//;
    $str =~ s/\s+$//;
    $str =~ s/[^a-z\d\s]//g;

    $str =~ s/\s\s+/ /g;
    $str =~ s/\s/_/g;

    $str =~ s/^(a|an|the)_(.*)$/$2_$1/;
    return $str;
}

sub _articleinfo {
    my $pkg     = shift;
    my $article = shift;

    my $info = $article->getChildrenByTagName(qq(articleinfo))->[0];
    
    if (! $info) {
	my $id = $article->getAttribute(qq(id));

	warn "can't get article info for article '$id'\n";
	return undef;
    }

    return $info;
}

sub _indexterm {
    my $pkg  = shift;
    my $info = shift;
    my $term = shift;

    return map { 
	$_->string_value();
    } $info->find(qq(indexterm[\@type='$term']/primary));
}

sub _keywords {
    my $pkg  = shift;
    my $info = shift;

    return map { 
	$_->string_value();
    } $info->find(qq(keywordset/keyword))->get_nodelist();
}

sub _author {
    my $pkg  = shift;
    my $info = shift;

    if (my $author = $info->find(qq(authorgroup/corpauthor))) {
	return $author;
    }

    return join(",",map { 
	join(" ",$_->find(qq(firstname)),$_->find(qq(surname)));
    } $info->find(qq(authorgroup/author))->get_nodelist());
}

sub _uid {
    my $pkg     = shift;
    my $article = shift;
    return $article->getAttribute(qq(id));
}

sub _articleid {
    my $self    = shift;
    my $article = shift;

    if (my $id = $article->find(qq(articleinfo/indexterm[\@type='articleid']))) {
	return $id->[0]->find(qq(primary));
    }

    return $self->_md5($self->_uid($article));
}

sub _cleanarticle {
    my $self    = shift;
    my $article = shift;

    return [ map {
	$_ =~ s/^\s+//; $_;
    } grep { 
	$_ =~ /\w/;
    } map {
	$_->textContent();
    } ($article->getChildrenByTagName(qq(para))) ];
}

sub _sectname {
    my $self    = shift;
    my $sr_name = shift;

    $$sr_name = lc($$sr_name);
    $$sr_name =~ s/ //g;
    $$sr_name =~ s/:/\//g;
}

sub _md5 {
    my $self = shift;
    return md5_hex($_[0]);
}

=head1 VERSION

1.3

=head1 DATE

$Date: 2004/07/23 12:59:59 $

=head1 AUTHOR

Aaron Straup Cope E<lt>ascope@cpan.orgE<gt>

=head1 LICENSE

Copyright (c) 2003-2004, Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under
the same terms as Perl itself.

=cut

return 1;

__END__
